@@ -67,11 +67,11 @@ class PartialRangeTest(TestCase):
     def test_empty_list(self):
         self.assertFalse(self.range.contains_product(self.prod))
 
-    def test_included_classes(self):
+    def _test_included_classes(self):
         self.range.classes.add(self.prod.product_class)
         self.assertTrue(self.range.contains_product(self.prod))
 
-    def test_included_class_with_exception(self):
+    def _test_included_class_with_exception(self):
         self.range.classes.add(self.prod.product_class)
         self.range.excluded_products.add(self.prod)
         self.assertFalse(self.range.contains_product(self.prod))
@@ -312,12 +312,47 @@ class AbsoluteDiscountBenefitTest(OfferTest):
         benefit = AbsoluteDiscountBenefit(range=rng, type='Absolute', value=Decimal('100'))
         self.assertTrue(condition.is_satisfied(basket))
         self.assertEquals(Decimal('100'), benefit.apply(basket, condition))
+        self.assertTrue(condition.is_satisfied(basket))
         self.assertEquals(Decimal('100'), benefit.apply(basket, condition))
+        self.assertTrue(condition.is_satisfied(basket))
         self.assertEquals(Decimal('100'), benefit.apply(basket, condition))
+        self.assertTrue(condition.is_satisfied(basket))
         self.assertEquals(Decimal('100'), benefit.apply(basket, condition))
+        self.assertTrue(condition.is_satisfied(basket))
+        self.assertEquals(Decimal('100'), benefit.apply(basket, condition))
+        self.assertFalse(condition.is_satisfied(basket))
+        self.assertEquals(Decimal('0'), benefit.apply(basket, condition))
+
+    def test_absolute_consumes_all(self):
+        product1 = create_product(Decimal('150'))
+        product2 = create_product(Decimal('300'))
+        product3 = create_product(Decimal('300'))
+        rng = Range.objects.create(name='Dummy')
+        rng.included_products.add(product1)
+        rng.included_products.add(product2)
+        rng.included_products.add(product3)
+        condition = ValueCondition(range=rng, type='Value', value=Decimal('500'))
+        basket = Basket.objects.create()
+        basket.add_product(product1, 1)
+        basket.add_product(product2, 1)
+        basket.add_product(product3, 1)
+        benefit = AbsoluteDiscountBenefit(range=rng, type='Absolute', value=Decimal('100'))
+        self.assertTrue(condition.is_satisfied(basket))
         self.assertEquals(Decimal('100'), benefit.apply(basket, condition))
         self.assertEquals(Decimal('0'), benefit.apply(basket, condition))
 
+    def test_absolute_applies_line_discount(self):
+        product = create_product(Decimal('500'))
+        rng = Range.objects.create(name='Dummy')
+        rng.included_products.add(product)
+        condition = ValueCondition(range=rng, type='Value', value=Decimal('500'))
+        basket = Basket.objects.create()
+        basket.add_product(product, 1)
+        benefit = AbsoluteDiscountBenefit(range=rng, type='Absolute', value=Decimal('100'))
+        self.assertTrue(condition.is_satisfied(basket))
+        self.assertEquals(Decimal('100'), benefit.apply(basket, condition))
+        self.assertEquals(Decimal('100'), basket.all_lines()[0]._discount)
+
 
 class MultibuyDiscountBenefitTest(OfferTest):
 
